@@ -1,11 +1,12 @@
 /* Hash tables.
-   Copyright (C) 2000, 2001 Free Software Foundation, Inc.
+   Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
+   2009, 2010, 2011 Free Software Foundation, Inc.
 
 This file is part of GNU Wget.
 
 GNU Wget is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2 of the License, or (at
+the Free Software Foundation; either version 3 of the License, or (at
 your option) any later version.
 
 GNU Wget is distributed in the hope that it will be useful,
@@ -14,57 +15,51 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with Wget; if not, write to the Free Software
-Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+along with Wget.  If not, see <http://www.gnu.org/licenses/>.
 
-In addition, as a special exception, the Free Software Foundation
-gives permission to link the code of its release of Wget with the
-OpenSSL project's "OpenSSL" library (or with modified versions of it
-that use the same license as the "OpenSSL" library), and distribute
-the linked executables.  You must obey the GNU General Public License
-in all respects for all of the code used other than "OpenSSL".  If you
-modify this file, you may extend this exception to your version of the
-file, but you are not obligated to do so.  If you do not wish to do
-so, delete this exception statement from your version.  */
+Additional permission under GNU GPL version 3 section 7
+
+If you modify this program, or any covered work, by linking or
+combining it with the OpenSSL project's OpenSSL library (or a
+modified version of that library), containing parts covered by the
+terms of the OpenSSL or SSLeay licenses, the Free Software Foundation
+grants you additional permission to convey the resulting work.
+Corresponding Source for a non-source form of such a combination
+shall include the source code for the parts of OpenSSL used as well
+as that of the covered work.  */
 
 /* With -DSTANDALONE, this file can be compiled outside Wget source
    tree.  To test, also use -DTEST.  */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-# ifdef HAVE_STRING_H
-#  include <string.h>
-# else
-#  include <strings.h>
-# endif
-# ifdef HAVE_LIMITS_H
-#  include <limits.h>
-# endif
-#else
-/* If running without Autoconf, go ahead and assume presence of
-   standard C89 headers.  */
-# include <string.h>
-# include <limits.h>
+#ifndef STANDALONE
+# include "wget.h"
 #endif
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <assert.h>
+#include <string.h>
+#include <limits.h>
 
 #ifndef STANDALONE
 /* Get Wget's utility headers. */
-# include "wget.h"
 # include "utils.h"
 #else
 /* Make do without them. */
 # define xnew(x) xmalloc (sizeof (x))
 # define xnew_array(type, x) xmalloc (sizeof (type) * (x))
-# define xmalloc malloc		/* or something that exits
-				   if not enough memory */
+# define xmalloc malloc
 # define xfree free
-# define countof(x) (sizeof (x) / sizeof ((x)[0]))
-# define TOLOWER(x) ('A' <= (x) && (x) <= 'Z' ? (x) - 32 : (x))
-# define PARAMS(x) x
+# ifndef countof
+#  define countof(x) (sizeof (x) / sizeof ((x)[0]))
+# endif
+# include <ctype.h>
+# define c_tolower(x) tolower ((unsigned char) (x))
+# ifdef HAVE_STDINT_H
+#  include <stdint.h>
+# else
+   typedef unsigned long uintptr_t;
+# endif
 #endif
 
 #include "hash.h"
@@ -84,8 +79,10 @@ so, delete this exception statement from your version.  */
      hash_table_get       -- retrieves value of key.
      hash_table_get_pair  -- get key/value pair for key.
      hash_table_contains  -- test whether the table contains key.
-     hash_table_remove    -- remove the key->value mapping for key.
-     hash_table_map       -- iterate through table mappings.
+     hash_table_remove    -- remove key->value mapping for given key.
+     hash_table_for_each  -- call function for each table entry.
+     hash_table_iterate   -- iterate over entries in hash table.
+     hash_table_iter_next -- return next element during iteration.
      hash_table_clear     -- clear hash table contents.
      hash_table_count     -- return the number of entries in the table.
 
@@ -94,33 +91,34 @@ so, delete this exception statement from your version.  */
    with each resize, which ensures that the amortized time per
    operation remains constant.
 
-   By default, tables created by hash_table_new consider the keys to
-   be equal if their pointer values are the same.  You can use
-   make_string_hash_table to create tables whose keys are considered
-   equal if their string contents are the same.  In the general case,
-   the criterion of equality used to compare keys is specified at
-   table creation time with two callback functions, "hash" and "test".
-   The hash function transforms the key into an arbitrary number that
-   must be the same for two equal keys.  The test function accepts two
-   keys and returns non-zero if they are to be considered equal.
+   If not instructed otherwise, tables created by hash_table_new
+   consider the keys to be equal if their pointer values are the same.
+   You can use make_string_hash_table to create tables whose keys are
+   considered equal if their string contents are the same.  In the
+   general case, the criterion of equality used to compare keys is
+   specified at table creation time with two callback functions,
+   "hash" and "test".  The hash function transforms the key into an
+   arbitrary number that must be the same for two equal keys.  The
+   test function accepts two keys and returns non-zero if they are to
+   be considered equal.
 
    Note that neither keys nor values are copied when inserted into the
    hash table, so they must exist for the lifetime of the table.  This
    means that e.g. the use of static strings is OK, but objects with a
-   shorter life-time need to be copied (with strdup() or the like in
-   the case of strings) before being inserted.  */
+   shorter life-time probably need to be copied (with strdup() or the
+   like in the case of strings) before being inserted.  */
 
 /* IMPLEMENTATION:
 
    The hash table is implemented as an open-addressed table with
    linear probing collision resolution.
 
-   The above means that all the hash entries (pairs of pointers, key
-   and value) are stored in a contiguous array.  The position of each
-   mapping is determined by the hash value of its key and the size of
-   the table: location := hash(key) % size.  If two different keys end
-   up on the same position (collide), the one that came second is
-   placed at the next empty position following the occupied place.
+   The above means that all the cells (each cell containing a key and
+   a value pointer) are stored in a contiguous array.  Array position
+   of each cell is determined by the hash value of its key and the
+   size of the table: location := hash(key) % size.  If two different
+   keys end up on the same position (collide), the one that came
+   second is stored in the first unoccupied cell that follows it.
    This collision resolution technique is called "linear probing".
 
    There are more advanced collision resolution methods (quadratic
@@ -130,13 +128,13 @@ so, delete this exception statement from your version.  */
    count/size ratio (fullness) is kept below 75%.  We make sure to
    grow and rehash the table whenever this threshold is exceeded.
 
-   Collisions make deletion tricky because clearing a position
-   followed by a colliding entry would make the position seem empty
-   and the colliding entry not found.  One solution is to leave a
-   "tombstone" instead of clearing the entry, and another is to
-   carefully rehash the entries immediately following the deleted one.
-   We use the latter method because it results in less bookkeeping and
-   faster retrieval at the (slight) expense of deletion.  */
+   Collisions complicate deletion because simply clearing a cell
+   followed by previously collided entries would cause those neighbors
+   to not be picked up by find_cell later.  One solution is to leave a
+   "tombstone" marker instead of clearing the cell, and another is to
+   recalculate the positions of adjacent cells.  We take the latter
+   approach because it results in less bookkeeping garbage and faster
+   retrieval at the (slight) expense of deletion.  */
 
 /* Maximum allowed fullness: when hash table's fullness exceeds this
    value, the table is resized.  */
@@ -147,32 +145,32 @@ so, delete this exception statement from your version.  */
    resizes.  */
 #define HASH_RESIZE_FACTOR 2
 
-struct mapping {
+struct cell {
   void *key;
   void *value;
 };
 
-typedef unsigned long (*hashfun_t) PARAMS ((const void *));
-typedef int (*testfun_t) PARAMS ((const void *, const void *));
+typedef unsigned long (*hashfun_t) (const void *);
+typedef int (*testfun_t) (const void *, const void *);
 
 struct hash_table {
   hashfun_t hash_function;
   testfun_t test_function;
 
-  struct mapping *mappings;	/* pointer to the table entries. */
-  int size;			/* size of the array. */
+  struct cell *cells;           /* contiguous array of cells. */
+  int size;                     /* size of the array. */
 
-  int count;			/* number of non-empty entries. */
-  int resize_threshold;		/* after size exceeds this number of
-				   entries, resize the table.  */
-  int prime_offset;		/* the offset of the current prime in
-				   the prime table. */
+  int count;                    /* number of occupied entries. */
+  int resize_threshold;         /* after size exceeds this number of
+                                   entries, resize the table.  */
+  int prime_offset;             /* the offset of the current prime in
+                                   the prime table. */
 };
 
 /* We use the all-bits-set constant (INVALID_PTR) marker to mean that
-   a mapping is empty.  It is unaligned and therefore illegal as a
-   pointer.  INVALID_PTR_BYTE (0xff) is the one-byte value used to
-   initialize the mappings array as empty.
+   a cell is empty.  It is unaligned and therefore illegal as a
+   pointer.  INVALID_PTR_CHAR (0xff) is the single-character constant
+   used to initialize the entire cells array as empty.
 
    The all-bits-set value is a better choice than NULL because it
    allows the use of NULL/0 keys.  Since the keys are either integers
@@ -180,23 +178,26 @@ struct hash_table {
    -1.  This is acceptable because it still allows the use of
    nonnegative integer keys.  */
 
-#define INVALID_PTR ((void *) ~(unsigned long)0)
+#define INVALID_PTR ((void *) ~(uintptr_t) 0)
 #ifndef UCHAR_MAX
 # define UCHAR_MAX 0xff
 #endif
-#define INVALID_PTR_BYTE UCHAR_MAX
+#define INVALID_PTR_CHAR UCHAR_MAX
 
-#define NON_EMPTY(mp) ((mp)->key != INVALID_PTR)
-#define MARK_AS_EMPTY(mp) ((mp)->key = INVALID_PTR)
+/* Whether the cell C is occupied (non-empty). */
+#define CELL_OCCUPIED(c) ((c)->key != INVALID_PTR)
 
-/* "Next" mapping is the mapping after MP, but wrapping back to
-   MAPPINGS when MP would reach MAPPINGS+SIZE.  */
-#define NEXT_MAPPING(mp, mappings, size) (mp != mappings + (size - 1)	\
-					  ? mp + 1 : mappings)
+/* Clear the cell C, i.e. mark it as empty (unoccupied). */
+#define CLEAR_CELL(c) ((c)->key = INVALID_PTR)
 
-/* Loop over non-empty mappings starting at MP. */
-#define LOOP_NON_EMPTY(mp, mappings, size)				\
-  for (; NON_EMPTY (mp); mp = NEXT_MAPPING (mp, mappings, size))
+/* "Next" cell is the cell following C, but wrapping back to CELLS
+   when C would reach CELLS+SIZE.  */
+#define NEXT_CELL(c, cells, size) (c != cells + (size - 1) ? c + 1 : cells)
+
+/* Loop over occupied cells starting at C, terminating the loop when
+   an empty cell is encountered.  */
+#define FOREACH_OCCUPIED_ADJACENT(c, cells, size)                               \
+  for (; CELL_OCCUPIED (c); c = NEXT_CELL (c, cells, size))
 
 /* Return the position of KEY in hash table SIZE large, hash function
    being HASHFUN.  */
@@ -225,24 +226,24 @@ prime_size (int size, int *prime_offset)
     243370577, 316381771, 411296309, 534685237, 695090819, 903618083,
     1174703521, 1527114613, 1837299131, 2147483647
   };
-  int i;
+  size_t i;
 
   for (i = *prime_offset; i < countof (primes); i++)
     if (primes[i] >= size)
       {
-	/* Set the offset to the next prime.  That is safe because,
-	   next time we are called, it will be with a larger SIZE,
-	   which means we could never return the same prime anyway.
-	   (If that is not the case, the caller can simply reset
-	   *prime_offset.)  */
-	*prime_offset = i + 1;
-	return primes[i];
+        /* Set the offset to the next prime.  That is safe because,
+           next time we are called, it will be with a larger SIZE,
+           which means we could never return the same prime anyway.
+           (If that is not the case, the caller can simply reset
+           *prime_offset.)  */
+        *prime_offset = i + 1;
+        return primes[i];
       }
 
   abort ();
 }
 
-static int cmp_pointer PARAMS ((const void *, const void *));
+static int cmp_pointer (const void *, const void *);
 
 /* Create a hash table with hash function HASH_FUNCTION and test
    function TEST_FUNCTION.  The table is empty (its count is 0), but
@@ -270,8 +271,8 @@ static int cmp_pointer PARAMS ((const void *, const void *));
 
 struct hash_table *
 hash_table_new (int items,
-		unsigned long (*hash_function) (const void *),
-		int (*test_function) (const void *, const void *))
+                unsigned long (*hash_function) (const void *),
+                int (*test_function) (const void *, const void *))
 {
   int size;
   struct hash_table *ht = xnew (struct hash_table);
@@ -291,11 +292,11 @@ hash_table_new (int items,
   ht->resize_threshold = size * HASH_MAX_FULLNESS;
   /*assert (ht->resize_threshold >= items);*/
 
-  ht->mappings = xnew_array (struct mapping, ht->size);
+  ht->cells = xnew_array (struct cell, ht->size);
 
-  /* Mark mappings as empty.  We use 0xff rather than 0 to mark empty
+  /* Mark cells as empty.  We use 0xff rather than 0 to mark empty
      keys because it allows us to use NULL/0 as keys.  */
-  memset (ht->mappings, INVALID_PTR_BYTE, size * sizeof (struct mapping));
+  memset (ht->cells, INVALID_PTR_CHAR, size * sizeof (struct cell));
 
   ht->count = 0;
 
@@ -307,26 +308,26 @@ hash_table_new (int items,
 void
 hash_table_destroy (struct hash_table *ht)
 {
-  xfree (ht->mappings);
+  xfree (ht->cells);
   xfree (ht);
 }
 
-/* The heart of most functions in this file -- find the mapping whose
-   KEY is equal to key, using linear probing.  Returns the mapping
-   that matches KEY, or the first empty mapping if none matches.  */
+/* The heart of most functions in this file -- find the cell whose
+   KEY is equal to key, using linear probing.  Returns the cell
+   that matches KEY, or the first empty cell if none matches.  */
 
-static inline struct mapping *
-find_mapping (const struct hash_table *ht, const void *key)
+static inline struct cell *
+find_cell (const struct hash_table *ht, const void *key)
 {
-  struct mapping *mappings = ht->mappings;
+  struct cell *cells = ht->cells;
   int size = ht->size;
-  struct mapping *mp = mappings + HASH_POSITION (key, ht->hash_function, size);
+  struct cell *c = cells + HASH_POSITION (key, ht->hash_function, size);
   testfun_t equals = ht->test_function;
 
-  LOOP_NON_EMPTY (mp, mappings, size)
-    if (equals (key, mp->key))
+  FOREACH_OCCUPIED_ADJACENT (c, cells, size)
+    if (equals (key, c->key))
       break;
-  return mp;
+  return c;
 }
 
 /* Get the value that corresponds to the key KEY in the hash table HT.
@@ -339,9 +340,9 @@ find_mapping (const struct hash_table *ht, const void *key)
 void *
 hash_table_get (const struct hash_table *ht, const void *key)
 {
-  struct mapping *mp = find_mapping (ht, key);
-  if (NON_EMPTY (mp))
-    return mp->value;
+  struct cell *c = find_cell (ht, key);
+  if (CELL_OCCUPIED (c))
+    return c->value;
   else
     return NULL;
 }
@@ -351,15 +352,15 @@ hash_table_get (const struct hash_table *ht, const void *key)
 
 int
 hash_table_get_pair (const struct hash_table *ht, const void *lookup_key,
-		     void *orig_key, void *value)
+                     void *orig_key, void *value)
 {
-  struct mapping *mp = find_mapping (ht, lookup_key);
-  if (NON_EMPTY (mp))
+  struct cell *c = find_cell (ht, lookup_key);
+  if (CELL_OCCUPIED (c))
     {
       if (orig_key)
-	*(void **)orig_key = mp->key;
+        *(void **)orig_key = c->key;
       if (value)
-	*(void **)value = mp->value;
+        *(void **)value = c->value;
       return 1;
     }
   else
@@ -371,8 +372,8 @@ hash_table_get_pair (const struct hash_table *ht, const void *lookup_key,
 int
 hash_table_contains (const struct hash_table *ht, const void *key)
 {
-  struct mapping *mp = find_mapping (ht, key);
-  return NON_EMPTY (mp);
+  struct cell *c = find_cell (ht, key);
+  return CELL_OCCUPIED (c);
 }
 
 /* Grow hash table HT as necessary, and rehash all the key-value
@@ -382,40 +383,40 @@ static void
 grow_hash_table (struct hash_table *ht)
 {
   hashfun_t hasher = ht->hash_function;
-  struct mapping *old_mappings = ht->mappings;
-  struct mapping *old_end      = ht->mappings + ht->size;
-  struct mapping *mp, *mappings;
+  struct cell *old_cells = ht->cells;
+  struct cell *old_end   = ht->cells + ht->size;
+  struct cell *c, *cells;
   int newsize;
 
   newsize = prime_size (ht->size * HASH_RESIZE_FACTOR, &ht->prime_offset);
 #if 0
   printf ("growing from %d to %d; fullness %.2f%% to %.2f%%\n",
-	  ht->size, newsize,
-	  100.0 * ht->count / ht->size,
-	  100.0 * ht->count / newsize);
+          ht->size, newsize,
+          100.0 * ht->count / ht->size,
+          100.0 * ht->count / newsize);
 #endif
 
   ht->size = newsize;
   ht->resize_threshold = newsize * HASH_MAX_FULLNESS;
 
-  mappings = xnew_array (struct mapping, newsize);
-  memset (mappings, INVALID_PTR_BYTE, newsize * sizeof (struct mapping));
-  ht->mappings = mappings;
+  cells = xnew_array (struct cell, newsize);
+  memset (cells, INVALID_PTR_CHAR, newsize * sizeof (struct cell));
+  ht->cells = cells;
 
-  for (mp = old_mappings; mp < old_end; mp++)
-    if (NON_EMPTY (mp))
+  for (c = old_cells; c < old_end; c++)
+    if (CELL_OCCUPIED (c))
       {
-	struct mapping *new_mp;
-	/* We don't need to test for uniqueness of keys because they
-	   come from the hash table and are therefore known to be
-	   unique.  */
-	new_mp = mappings + HASH_POSITION (mp->key, hasher, newsize);
-	LOOP_NON_EMPTY (new_mp, mappings, newsize)
-	  ;
-	*new_mp = *mp;
+        struct cell *new_c;
+        /* We don't need to test for uniqueness of keys because they
+           come from the hash table and are therefore known to be
+           unique.  */
+        new_c = cells + HASH_POSITION (c->key, hasher, newsize);
+        FOREACH_OCCUPIED_ADJACENT (new_c, cells, newsize)
+          ;
+        *new_c = *c;
       }
 
-  xfree (old_mappings);
+  xfree (old_cells);
 }
 
 /* Put VALUE in the hash table HT under the key KEY.  This regrows the
@@ -424,12 +425,12 @@ grow_hash_table (struct hash_table *ht)
 void
 hash_table_put (struct hash_table *ht, const void *key, void *value)
 {
-  struct mapping *mp = find_mapping (ht, key);
-  if (NON_EMPTY (mp))
+  struct cell *c = find_cell (ht, key);
+  if (CELL_OCCUPIED (c))
     {
       /* update existing item */
-      mp->key   = (void *)key; /* const? */
-      mp->value = value;
+      c->key   = (void *)key; /* const? */
+      c->value = value;
       return;
     }
 
@@ -438,58 +439,58 @@ hash_table_put (struct hash_table *ht, const void *key, void *value)
   if (ht->count >= ht->resize_threshold)
     {
       grow_hash_table (ht);
-      mp = find_mapping (ht, key);
+      c = find_cell (ht, key);
     }
 
   /* add new item */
   ++ht->count;
-  mp->key   = (void *)key;	/* const? */
-  mp->value = value;
+  c->key   = (void *)key;       /* const? */
+  c->value = value;
 }
 
-/* Remove a mapping that matches KEY from HT.  Return 0 if there was
-   no such entry; return 1 if an entry was removed.  */
+/* Remove KEY->value mapping from HT.  Return 0 if there was no such
+   entry; return 1 if an entry was removed.  */
 
 int
 hash_table_remove (struct hash_table *ht, const void *key)
 {
-  struct mapping *mp = find_mapping (ht, key);
-  if (!NON_EMPTY (mp))
+  struct cell *c = find_cell (ht, key);
+  if (!CELL_OCCUPIED (c))
     return 0;
   else
     {
       int size = ht->size;
-      struct mapping *mappings = ht->mappings;
+      struct cell *cells = ht->cells;
       hashfun_t hasher = ht->hash_function;
 
-      MARK_AS_EMPTY (mp);
+      CLEAR_CELL (c);
       --ht->count;
 
-      /* Rehash all the entries following MP.  The alternative
-	 approach is to mark the entry as deleted, i.e. create a
-	 "tombstone".  That speeds up removal, but leaves a lot of
-	 garbage and slows down hash_table_get and hash_table_put.  */
+      /* Rehash all the entries following C.  The alternative
+         approach is to mark the entry as deleted, i.e. create a
+         "tombstone".  That speeds up removal, but leaves a lot of
+         garbage and slows down hash_table_get and hash_table_put.  */
 
-      mp = NEXT_MAPPING (mp, mappings, size);
-      LOOP_NON_EMPTY (mp, mappings, size)
-	{
-	  const void *key2 = mp->key;
-	  struct mapping *mp_new;
+      c = NEXT_CELL (c, cells, size);
+      FOREACH_OCCUPIED_ADJACENT (c, cells, size)
+        {
+          const void *key2 = c->key;
+          struct cell *c_new;
 
-	  /* Find the new location for the key. */
-	  mp_new = mappings + HASH_POSITION (key2, hasher, size);
-	  LOOP_NON_EMPTY (mp_new, mappings, size)
-	    if (key2 == mp_new->key)
-	      /* The mapping MP (key2) is already where we want it (in
-		 MP_NEW's "chain" of keys.)  */
-	      goto next_rehash;
+          /* Find the new location for the key. */
+          c_new = cells + HASH_POSITION (key2, hasher, size);
+          FOREACH_OCCUPIED_ADJACENT (c_new, cells, size)
+            if (key2 == c_new->key)
+              /* The cell C (key2) is already where we want it (in
+                 C_NEW's "chain" of keys.)  */
+              goto next_rehash;
 
-	  *mp_new = *mp;
-	  MARK_AS_EMPTY (mp);
+          *c_new = *c;
+          CLEAR_CELL (c);
 
-	next_rehash:
-	  ;
-	}
+        next_rehash:
+          ;
+        }
       return 1;
     }
 }
@@ -501,41 +502,84 @@ hash_table_remove (struct hash_table *ht, const void *key)
 void
 hash_table_clear (struct hash_table *ht)
 {
-  memset (ht->mappings, INVALID_PTR_BYTE, ht->size * sizeof (struct mapping));
+  memset (ht->cells, INVALID_PTR_CHAR, ht->size * sizeof (struct cell));
   ht->count = 0;
 }
 
-/* Map MAPFUN over all the mappings in hash table HT.  MAPFUN is
-   called with three arguments: the key, the value, and MAPARG.
+/* Call FN for each entry in HT.  FN is called with three arguments:
+   the key, the value, and ARG.  When FN returns a non-zero value, the
+   mapping stops.
 
    It is undefined what happens if you add or remove entries in the
-   hash table while hash_table_map is running.  The exception is the
-   entry you're currently mapping over; you may remove or change that
-   entry.  */
+   hash table while hash_table_for_each is running.  The exception is
+   the entry you're currently mapping over; you may call
+   hash_table_put or hash_table_remove on that entry's key.  That is
+   also the reason why this function cannot be implemented in terms of
+   hash_table_iterate.  */
 
 void
-hash_table_map (struct hash_table *ht,
-		int (*mapfun) (void *, void *, void *),
-		void *maparg)
+hash_table_for_each (struct hash_table *ht,
+                     int (*fn) (void *, void *, void *), void *arg)
 {
-  struct mapping *mp  = ht->mappings;
-  struct mapping *end = ht->mappings + ht->size;
+  struct cell *c = ht->cells;
+  struct cell *end = ht->cells + ht->size;
 
-  for (; mp < end; mp++)
-    if (NON_EMPTY (mp))
+  for (; c < end; c++)
+    if (CELL_OCCUPIED (c))
       {
-	void *key;
+        void *key;
       repeat:
-	key = mp->key;
-	if (mapfun (key, mp->value, maparg))
-	  return;
-	/* hash_table_remove might have moved the adjacent
-	   mappings. */
-	if (mp->key != key && NON_EMPTY (mp))
-	  goto repeat;
+        key = c->key;
+        if (fn (key, c->value, arg))
+          return;
+        /* hash_table_remove might have moved the adjacent cells. */
+        if (c->key != key && CELL_OCCUPIED (c))
+          goto repeat;
       }
 }
 
+/* Initiate iteration over HT.  Entries are obtained with
+   hash_table_iter_next, a typical iteration loop looking like this:
+
+       hash_table_iterator iter;
+       for (hash_table_iterate (ht, &iter); hash_table_iter_next (&iter); )
+         ... do something with iter.key and iter.value ...
+
+   The iterator does not need to be deallocated after use.  The hash
+   table must not be modified while being iterated over.  */
+
+void
+hash_table_iterate (struct hash_table *ht, hash_table_iterator *iter)
+{
+  iter->pos = ht->cells;
+  iter->end = ht->cells + ht->size;
+}
+
+/* Get the next hash table entry.  ITER is an iterator object
+   initialized using hash_table_iterate.  While there are more
+   entries, the key and value pointers are stored to ITER->key and
+   ITER->value respectively and 1 is returned.  When there are no more
+   entries, 0 is returned.
+
+   If the hash table is modified between calls to this function, the
+   result is undefined.  */
+
+int
+hash_table_iter_next (hash_table_iterator *iter)
+{
+  struct cell *c = iter->pos;
+  struct cell *end = iter->end;
+  for (; c < end; c++)
+    if (CELL_OCCUPIED (c))
+      {
+        iter->key = c->key;
+        iter->value = c->value;
+        iter->pos = c + 1;
+        return 1;
+      }
+  return 0;
+}
+
 /* Return the number of elements in the hash table.  This is not the
    same as the physical size of the hash table, which is always
    greater than the number of elements.  */
@@ -566,7 +610,7 @@ hash_table_count (const struct hash_table *ht)
      a hash function that returns 0 for any given object is a
      perfectly valid (albeit extremely bad) hash function.  A hash
      function that hashes a string by adding up all its characters is
-     another example of a valid (but quite bad) hash function.
+     another example of a valid (but still quite bad) hash function.
 
      It is not hard to make hash and test functions agree about
      equality.  For example, if the test function compares strings
@@ -574,35 +618,36 @@ hash_table_count (const struct hash_table *ht)
      characters when calculating the hash value.  That ensures that
      two strings differing only in case will hash the same.
 
-   - If you care about performance, choose a hash function with as
-     good "spreading" as possible.  A good hash function will use all
-     the bits of the input when calculating the hash, and will react
-     to even small changes in input with a completely different
-     output.  Finally, don't make the hash function itself overly
-     slow, because you'll be incurring a non-negligible overhead to
-     all hash table operations.  */
+   - To prevent performance degradation, choose a hash function with
+     as good "spreading" as possible.  A good hash function will use
+     all the bits of the input when calculating the hash, and will
+     react to even small changes in input with a completely different
+     output.  But don't make the hash function itself overly slow,
+     because you'll be incurring a non-negligible overhead to all hash
+     table operations.  */
 
 /*
  * Support for hash tables whose keys are strings.
  *
  */
-   
-/* 31 bit hash function.  Taken from Gnome's glib, modified to use
+
+/* Base 31 hash function.  Taken from Gnome's glib, modified to use
    standard C types.
 
    We used to use the popular hash function from the Dragon Book, but
-   this one seems to perform much better.  */
+   this one seems to perform much better, both by being faster and by
+   generating less collisions.  */
 
 static unsigned long
 hash_string (const void *key)
 {
   const char *p = key;
   unsigned int h = *p;
-  
+
   if (h)
     for (p += 1; *p != '\0'; p++)
       h = (h << 5) - h + *p;
-  
+
   return h;
 }
 
@@ -635,12 +680,12 @@ static unsigned long
 hash_string_nocase (const void *key)
 {
   const char *p = key;
-  unsigned int h = TOLOWER (*p);
-  
+  unsigned int h = c_tolower (*p);
+
   if (h)
     for (p += 1; *p != '\0'; p++)
-      h = (h << 5) - h + TOLOWER (*p);
-  
+      h = (h << 5) - h + c_tolower (*p);
+
   return h;
 }
 
@@ -664,14 +709,15 @@ make_nocase_string_hash_table (int items)
 /* Hashing of numeric values, such as pointers and integers.
 
    This implementation is the Robert Jenkins' 32 bit Mix Function,
-   with a simple adaptation for 64-bit values.  It offers excellent
-   spreading of values and doesn't need to know the hash table size to
-   work (unlike the very popular Knuth's multiplication hash).  */
+   with a simple adaptation for 64-bit values.  According to Jenkins
+   it should offer excellent spreading of values.  Unlike the popular
+   Knuth's multiplication hash, this function doesn't need to know the
+   hash table size to work.  */
 
 unsigned long
 hash_pointer (const void *ptr)
 {
-  unsigned long key = (unsigned long)ptr;
+  uintptr_t key = (uintptr_t) ptr;
   key += (key << 12);
   key ^= (key >> 22);
   key += (key << 4);
@@ -680,7 +726,7 @@ hash_pointer (const void *ptr)
   key ^= (key >> 2);
   key += (key << 7);
   key ^= (key >> 12);
-#if SIZEOF_LONG > 4
+#if SIZEOF_VOID_P > 4
   key += (key << 44);
   key ^= (key >> 54);
   key += (key << 36);
@@ -690,7 +736,7 @@ hash_pointer (const void *ptr)
   key += (key << 39);
   key ^= (key >> 44);
 #endif
-  return key;
+  return (unsigned long) key;
 }
 
 static int
@@ -704,20 +750,16 @@ cmp_pointer (const void *ptr1, const void *ptr2)
 #include <stdio.h>
 #include <string.h>
 
-int
-print_hash_table_mapper (void *key, void *value, void *count)
-{
-  ++*(int *)count;
-  printf ("%s: %s\n", (const char *)key, (char *)value);
-  return 0;
-}
-
 void
 print_hash (struct hash_table *sht)
 {
-  int debug_count = 0;
-  hash_table_map (sht, print_hash_table_mapper, &debug_count);
-  assert (debug_count == sht->count);
+  hash_table_iterator iter;
+  int count = 0;
+
+  for (hash_table_iterate (sht, &iter); hash_table_iter_next (&iter);
+       ++count)
+    printf ("%s: %s\n", iter.key, iter.value);
+  assert (count == sht->count);
 }
 
 int
@@ -729,20 +771,20 @@ main (void)
     {
       int len = strlen (line);
       if (len <= 1)
-	continue;
+        continue;
       line[--len] = '\0';
       if (!hash_table_contains (ht, line))
-	hash_table_put (ht, strdup (line), "here I am!");
+        hash_table_put (ht, strdup (line), "here I am!");
 #if 1
       if (len % 5 == 0)
-	{
-	  char *line_copy;
-	  if (hash_table_get_pair (ht, line, &line_copy, NULL))
-	    {
-	      hash_table_remove (ht, line);
-	      xfree (line_copy);
-	    }
-	}
+        {
+          char *line_copy;
+          if (hash_table_get_pair (ht, line, &line_copy, NULL))
+            {
+              hash_table_remove (ht, line);
+              xfree (line_copy);
+            }
+        }
 #endif
     }
 #if 0
